@@ -51,8 +51,9 @@ atRisk <- function(x,
                    adjust.labels=TRUE,
                    ...){
     if (missing(times)) times <- seq(0,x$maxtime,x$maxtime/10)
-    if (x$model=="competing.risks")
-        px <- lifeTab(object=x,times=times,newdata=newdata,stats=NULL)[[1]]
+    if (x$model=="competing.risks"){
+        px <- lifeTab(object=x,times=times,cause=1,newdata=newdata,stats=NULL)[[1]]
+    }
     else if (x$model=="survival"){
         px <- lifeTab(object=x,times=times,newdata=newdata,stats=NULL)
     }
@@ -60,15 +61,15 @@ atRisk <- function(x,
         sumx <- lapply(data.frame(px)[,grep("n.risk",colnames(px)),drop=FALSE],function(x)x)
     else
         sumx <- lapply(px,function(v){
-            u <- v[,grep("n.risk",colnames(v)),drop=FALSE]
-            if (NCOL(u)>1){
-                ulist <- lapply(1:NCOL(u),function(i)u[,i])
-                names(ulist) <- colnames(u)
-                ulist
-            }
-            else
-                u
-        })
+                           u <- v[,grep("n.risk",colnames(v)),drop=FALSE]
+                           if (NCOL(u)>1){
+                               ulist <- lapply(1:NCOL(u),function(i)u[,i])
+                               names(ulist) <- colnames(u)
+                               ulist
+                           }
+                           else
+                               u
+                       })
     if (is.list(sumx[[1]]))
         sumx <- unlist(sumx,recursive=FALSE)
     if (all(sapply(sumx,NCOL))==1)
@@ -92,58 +93,58 @@ atRisk <- function(x,
     if (is.null(titlecol)){
         tcol <- 1
     } else {
-        if (is.na(titlecol[1]))
-            tcol <- 1
-        else
-            tcol <- titlecol[1]
-    }
+          if (is.na(titlecol[1]))
+              tcol <- 1
+          else
+              tcol <- titlecol[1]
+      }
     ##
     if (!is.null(title))
         mtext(title,
-              side=1,
-              at=pos,
-              col=tcol,
-              line=line[1]-1,
-              adj=adj,
-              cex=cex,
-              outer=FALSE,
-              xpd=NA,
-              ...)
+                        side=1,
+                        at=pos,
+                        col=tcol,
+                        line=line[1]-1,
+                        adj=adj,
+                        cex=cex,
+                        outer=FALSE,
+                        xpd=NA,
+                        ...)
     # labeling the no. at-risk below plot
     # --------------------------------------------------------------------
     ## if (is.null(adjust.labels) || adjust.labels==TRUE){
     ## labels <- format(labels,justify="left")}
     if (length(col)==nlines/2) ## 1 cluster level
-        col <- rep(col,rep(nlines/2,length(col)))
+        col <- rep(col,rep(2,length(col)))
     lapply(1:nlines,function(y){
-        mtext(text=as.character(sumx[[y]]),
-              side=1,
-              at=times,
-              line=rep(line[y],length(times)),
-              col=rep(col[y],length(times)),
-              cex=cex,
-              outer=FALSE,
-              xpd=NA,
-              ...)
-        if (is.null(labelcol)){
-            lcol <- col[y]
-        } else {
-            if (is.na(labelcol[y]))
-                lcol <- labelcol[1]
-            else
-                lcol <- labelcol[y]
-        }
-        ## print(labels[y])
-        mtext(text=labels[y],
-              side=1,
-              at=pos,
-              col=labelcol[y],
-              ## col=1,
-              line=line[y],
-              adj=adj,
-              cex=cex,
-              outer=FALSE,
-              xpd=NA,
-              ...)
-    })
+               mtext(text=as.character(sumx[[y]]),
+                               side=1,
+                               at=times,
+                               line=rep(line[y],length(times)),
+                               col=rep(col[y],length(times)),
+                               cex=cex,
+                               outer=FALSE,
+                               xpd=NA,
+                               ...)
+               if (is.null(labelcol)){
+                   lcol <- col[y]
+               } else {
+                     if (is.na(labelcol[y]))
+                         lcol <- labelcol[1]
+                     else
+                         lcol <- labelcol[y]
+                 }
+               ## print(labels[y])
+               mtext(text=labels[y],
+                               side=1,
+                               at=pos,
+                               col=labelcol[y],
+                               ## col=1,
+                               line=line[y],
+                               adj=adj,
+                               cex=cex,
+                               outer=FALSE,
+                               xpd=NA,
+                               ...)
+           })
 }
